from .OPCUAMachine import SubscriptionWrapper
from .Mixingpump import Mixingpump

class Smp(Mixingpump):
    """
    SMP

    OPC-UA client class for m-tec SMP machines (Mixingpump).
    Inherits from Mixingpump.
    """
    
    @property
    def s_rotaryvalve(self) -> bool:
        """
        bool: True if the rotary valve is running in automatic mode.
        """
        return self.safe_read("aut_cw", False)

    @s_rotaryvalve.setter
    def s_rotaryvalve(self, callback: callable):
        """
        Create a subscription for the rotary valve state (automatic mode).

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("aut_cw", callback)
    
    @property
    def s_compressor(self) -> bool:
        """
        bool: True if the compressor is running in automatic mode.
        """
        return self.safe_read("aut_comp", False)

    @s_compressor.setter
    def s_compressor(self, callback: callable):
        """
        Create a subscription for the compressor state (automatic mode).

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("aut_comp", callback)
    
    @property
    def s_vibrator_1(self) -> bool:
        """
        bool: True if vibrator 1 is running in automatic mode.
        """
        return self.safe_read("aut_vib_1", False)

    @s_vibrator_1.setter
    def s_vibrator_1(self, callback: callable):
        """
        Create a subscription for the vibrator 1 state (automatic mode).

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("aut_vib_1", callback)
    
    @property
    def s_vibrator_2(self) -> bool:
        """
        bool: True if vibrator 2 is running in automatic mode.
        """
        return self.safe_read("aut_vib_2", False)
    
    @s_vibrator_2.setter
    def s_vibrator_2(self, callback: callable):
        """
        Create a subscription for the vibrator 2 state (automatic mode).

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("aut_vib_2", callback)
    
    @property
    def m_silolevel(self) -> float:
        """
        float: Silo level in percentage (0-100%).
        """
        return self.safe_read("Silo_Level", 0.0)

    @m_silolevel.setter
    def m_silolevel(self, callback: callable):
        """
        Create a subscription for the silo level in percentage.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("Silo_Level", callback)


from .OPCUAMachine import OPCUAMachine, SubscriptionWrapper

class Dosingpump(OPCUAMachine):
    """
    Class for controlling a dosing pump via OPC-UA.
    Inherits from OPCUAMachine.
    """
    @property
    def run(self) -> bool:
        """
        bool: True if the dosingpump is set to run, False otherwise.
        """
        return self.read("state_dosingpump_on")
    @run.setter
    def run(self, state: bool):
        """
        Set the running state of the dosingpump.

        Args:
            state (bool): True to start, False to stop.
        """
        self.change("state_dosingpump_on", state, "bool")

    @property
    def s_running(self) -> bool:
        """
        bool: True if the dosingpump is running, False otherwise.
        """
        return self.read("state_fc_dosingpump")

    @s_running.setter
    def s_running(self, callback: callable):
        """
        Create a subscription for the running state of the dosingpump.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("state_fc_dosingpump", callback)

    @property
    def speed(self) -> float:
        """
        float: Speed setting of the dosingpump in ml/min.
        """
        return self.read("set_value_dosingpump")
    @speed.setter
    def speed(self, speed: float):
        """
        Set the speed of the dosingpump.

        Args:
            speed (float): Speed in ml/min.
        """
        self.change("set_value_dosingpump", speed, "float")

    @property
    def m_speed(self) -> int:
        """
        int: Real speed of the dosingpump in ml/min.
        """
        return self.read("actual_value_additive")

    @m_speed.setter
    def m_speed(self, callback: callable):
        """
        Create a subscription for the real speed of the dosingpump in ml/min.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("actual_value_additive", callback)

    @property
    def m_pumpspeed(self) -> float:
        """
        float: Real speed of the dosingpump in %.
        """
        return self.read("actual_value_dosingpump")

    @m_pumpspeed.setter
    def m_pumpspeed(self, callback: callable):
        """
        Create a subscription for the real speed of the dosingpump in %.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("actual_value_dosingpump", callback)

    @property
    def m_pressure(self) -> float:
        """
        float: Real pressure of the dosingpump in bar.
        """
        return self.read("actual_value_pressure_dosingpump")

    @m_pressure.setter
    def m_pressure(self, callback: callable):
        """
        Create a subscription for the real pressure of the dosingpump in bar.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("actual_value_pressure_dosingpump", callback)

    @property
    def cleaning(self) -> bool:
        """
        bool: True if cleaning water is running, False otherwise.
        """
        return self.read("state_solenoid_valve")
    @cleaning.setter
    def cleaning(self, state: bool):
        """
        Set the cleaning water state.

        Args:
            state (bool): True to start, False to stop.
        """
        self.change("state_solenoid_valve", state, "bool")

    @property
    def s_error(self) -> bool:
        """
        bool: True if the dosingpump is in error state.
        """
        return self.read("error_dosingpump")

    @s_error.setter
    def s_error(self, callback: callable):
        """
        Create a subscription for the error state of the dosingpump.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("error_dosingpump", callback)

    @property
    def s_error_no(self) -> int:
        """
        int: Error number of the dosingpump (0 = none).
        """
        return self.read("error_no_dosingpump")

    @s_error_no.setter
    def s_error_no(self, callback: callable):
        """
        Create a subscription for the error number of the dosingpump.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("error_no_dosingpump", callback)

    @property
    def s_ready(self) -> bool:
        """
        bool: True if the dosingpump is ready for operation.
        """
        return self.read("Ready_for_operation_dosingpump")

    @s_ready.setter
    def s_ready(self, callback: callable):
        """
        Create a subscription for the ready state of the dosingpump.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("Ready_for_operation_dosingpump", callback)
    
    @property
    def s_emergency_stop(self) -> bool:
        """
        bool: True if emergency stop is ok, False otherwise.
        """
        return bool(self.safe_read("emergency_stop_ok", False))

    @s_emergency_stop.setter
    def s_emergency_stop(self, callback: callable):
        """
        Create a subscription for the emergency stop state.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("emergency_stop_ok", callback)

    @property
    def s_on(self) -> bool:
        """
        bool: True if the machine is powered on, False otherwise.
        """
        return bool(self.safe_read("state_machine_on", False))

    @s_on.setter
    def s_on(self, callback: callable):
        """
        Create a subscription for the machine power state.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("state_machine_on", callback)
    
    @property
    def s_remote(self) -> bool:
        """
        bool: True if remote is connected.
        """
        return self.read("Remote_connected_dosingpump")

    @s_remote.setter
    def s_remote(self, callback: callable):
        """
        Create a subscription for the remote connection state.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        self.easy_subscribe("Remote_connected_dosingpump", callback)
    
    @property
    def s_fc(self) -> bool:
        """
        bool: True if frequency converter is ok, False otherwise.
        """
        return not bool(self.safe_read("state_fc_error_dosingpump", True)) # Inverted

    @s_fc.setter
    def s_fc(self, callback: callable):
        """
        Create a subscription for the frequency converter state.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        def cb(value, parameter):
            sw = SubscriptionWrapper(callback, subscription)
            sw.trigger(value=not value, parameter=parameter)
        subscription = self.easy_subscribe("state_fc_error_dosingpump", cb, False)
    
    @property
    def s_operating_pressure(self) -> bool:
        """
        bool: True if operating pressure is ok, False otherwise.
        """
        return not bool(self.safe_read("state_pressure_error_dosingpump", True)) # Inverted

    @s_operating_pressure.setter
    def s_operating_pressure(self, callback: callable):
        """
        Create a subscription for the operating pressure state.

        Args:
            callback (callable): Callback function (optional parameters: 'value', 'parameter' and 'subscription').
        """
        if not callable(callback):
            raise ValueError("Callback is not callable.")
        def cb(value, parameter):
            sw = SubscriptionWrapper(callback, subscription)
            sw.trigger(value=not value, parameter=parameter)
        subscription = self.easy_subscribe("state_pressure_error_dosingpump", cb, False)


from asyncua.sync import Client #https://github.com/FreeOpcUa/asyncua
from asyncua import ua #https://github.com/FreeOpcUa/asyncua

import inspect
from typing import Optional, Callable, Any, Union
import threading

class OPCUAMachine:
    """
    Base class for OPC-UA machine communication.

    Args:
        ip (str): IP address of the machine.
        baseNode (str): Base node of the machine.
    """
    def __init__(self, baseNode = "ns=4;s=|var|B-Fortis CC-Slim S04.Application.GVL_OPC.", livebitNode = "Livebit2machine"):
        self._baseNode = baseNode
        self._liveBitNode = livebitNode
        self._connected = False

    def connect(self, ip: str):
        """
        Connects to the machine using the provided IP address.

        Args:
            ip (str): IP address of the machine.
        """
        self._ip = ip
        if not self._ip:
            raise ValueError("No IP address provided.")
        
        # check if ip address starts with opc.tcp://, if not add it
        if not self._ip.startswith("opc.tcp://"):
            self._ip = "opc.tcp://" + self._ip

        # check if ip address has port, if not add default port 4840
        if ":" not in self._ip.split("//")[1]:
            self._ip += ":4840"
        
        self._reader = Client(url=self._ip)
        self._writer = Client(url=self._ip)
        self._reader.connect()
        self._writer.connect()
        self._reader.load_data_type_definitions()
        self._writer.load_data_type_definitions()

        self._connected = True

        # check if Livebit2machine exists, otherwise use Livebit2DuoMix
        try:
            self.read(self._liveBitNode)
        except ua.UaError:
            self._liveBitNode = "Livebit2DuoMix"
        # check if Livebit2DuoMix exists, otherwise throw error
        try:
            self.read(self._liveBitNode)
        except ua.UaError:
            raise ValueError("Livebit node not found. Machine not supported.")

        self.subscribe("Livebit2extern", self.changeLivebit, 500)
        
    
    def disconnect(self):
        """
        Disconnects from the machine.
        """
        self._reader.disconnect()
        self._writer.disconnect()
        self._connected = False

    def _status_change_callback(self, status):
        """
        Callback for subscription status changes.

        Args:
            status: The new status.
        """
        print("Connection status changed:", status)
        self._connected = status == ua.StatusCode(0)

    def safe_change(self, parameter: str, value: Any, typ: str) -> bool:
        """
        Changes the value of an OPC-UA variable, with feature-not-supported feedback.

        Args:
            parameter (str): Variable to change.
            value (Any): Value to change the variable to.
            typ (str): String of variable type ("bool", "uint16", "int32", "float").

        Returns:
            bool: True if successful, False if not supported.
        """
        try:
            self.change(parameter, value, typ)
            return True
        except KeyError:
            print(f"Feature '{parameter}' is not supported on this machine.")
            return False
    
    def safe_read(self, parameter: str, default: Any) -> Any:
        """
        Reads the value of an OPC-UA variable, with feature-not-supported feedback.

        Args:
            parameter (str): Variable to read.
            default: Value to return if parameter is not available.

        Returns:
            Any: Value of the variable or default if not available.
        """
        try:
            return self.read(parameter)
        except KeyError:
            print(f"Feature '{parameter}' is not supported on this machine.")
            return default
    
    def change(self, parameter: str, value: Any, typ: str):
        """
        Changes the value of an OPC-UA variable.

        Args:
            parameter (str): Variable to change.
            value (Any): Value to change the variable to.
            typ (str): String of variable type ("bool", "uint16", "int32", "float").
        """
        if not self._connected:
            raise ua.UaError("Not connected to machine.")
        
        if callable(value):
            self.easy_subscribe(parameter, value)
            return

        if typ == "bool":
            t = ua.VariantType.Boolean
            value = bool(value)
        elif typ == "uint16":
            t = ua.VariantType.UInt16
            value = int(abs(value))
        elif typ == "int32":
            t = ua.VariantType.Int32
            value = int(value)
        elif typ == "float":
            t = ua.VariantType.Float
            value = float(value)
        else:
            return
        node = self._writer.get_node(self._baseNode + parameter)
        node.set_value(ua.Variant(value, t))

    def read(self, parameter: str) -> Any:
        """
        Reads the value of an OPC-UA variable.

        Args:
            parameter (str): Variable to read.

        Returns:
            Any: Value of the variable.
        """
        node = self._reader.get_node(self._baseNode + parameter)
        return node.get_value()

    def easy_subscribe(self, parameter: Union[str, list], callback: Callable, wrap: bool = True, interval: int = 500):
        """
        Easy subscription method for a given OPC-UA parameter.

        Args:
            parameter (Union[str, list]): The OPC-UA parameter(s) to subscribe to.
            callback (Callable): Callback function receiving value and parameter.
            wrap (bool): Whether to wrap the callback to filter arguments. Default is True.
            interval (int): Interval in ms for checking the parameter.

        Returns:
            list: List of subscriptions created.
        """
                
        subscriptions = []
        handler = None
        if wrap:
            sw = SubscriptionWrapper(callback)
        # check if parameter is a string or an array of strings
        if isinstance(parameter, str):
            if wrap:
                subscription, handler = self.subscribe(parameter, sw.trigger)
                sw.subscription(subscription)
            else:
                subscription, handler = self.subscribe(parameter, callback)
                subscriptions.append(subscription)
        elif isinstance(parameter, list):
            for param in parameter:
                if isinstance(param, str):
                    if wrap:
                        subscription, handler = self.subscribe(param, sw.trigger)
                        sw.subscription(subscription)
                    else:
                        subscription, handler = self.subscribe(param, callback)
                        subscriptions.append(subscription)
        return subscriptions

    def subscribe(self, parameter: str, callback: Callable, interval: int = 500):
        """
        Subscribes to a given OPC-UA parameter.

        Args:
            parameter (str): The OPC-UA parameter to subscribe to.
            callback (Callable): Callback function receiving value and parameter.
            interval (int): Interval in ms for checking the parameter.

        Returns:
            list: [subscription, handler]
        """
        if not self._connected:
            raise ua.UaError("Not connected to machine.")
        
        subscriptionHandler = OpcuaSubscriptionHandler(parameter, callback, self._status_change_callback)
        subscription = self._reader.create_subscription(interval, subscriptionHandler)
        handler = subscription.subscribe_data_change(self._reader.get_node(self._baseNode + parameter))
        return [subscription, handler]

    def changeLivebit(self, value: bool, parameter=None):
        """
        Changes the Livebit value.

        Args:
            value (bool): The value to change the Livebit to.
            parameter: Unused, for callback compatibility.
        """
        if not self._connected:
            raise ua.UaError("Not connected to machine.")
        
        self.change(self._liveBitNode, value, "bool")

class SubscriptionWrapper:
    def __init__(self, callback: Callable, subscription=None):
        self._callback = callback
        self._subscriptions = subscription if subscription is not None else []

    def subscription(self, subscription):
        self._subscriptions.append(subscription)

    def delete(self):
        for subscription in self._subscriptions:
            subscription.delete()
        self._subscriptions = []

    def trigger(self, value, parameter):
        self.exec(value=value, parameter=parameter, subscription=self)

    def exec(self, **kwargs):
        sig = inspect.signature(self._callback)
        filtered_kwargs = {k: v for k, v in kwargs.items() if k in sig.parameters}
        # execute the callback in a new thread to avoid blocking
        threading.Thread(target=self._callback, kwargs=filtered_kwargs).start()

class OpcuaSubscriptionHandler:
    """
    Handler for OPC-UA subscription data changes.
    """
    def __init__(self, parameter, callback, status_change_callback=None):
        """
        Args:
            parameter (str): The parameter being subscribed to.
            callback (Callable): Callback function for data changes.
            status_change_callback (Callable, optional): Callback for status changes. Defaults to None.
        """
        self.parameter = parameter
        self.callback = callback
        self.status_change_callback = status_change_callback

    def datachange_notification(self, node, value, data):
        """
        Called when a data change notification is received.

        Args:
            node: The OPC-UA node.
            value: The new value.
            data: Additional data.
        """
        self.callback(value, self.parameter)

    def status_change_notification(self, status):
        """
        Called when a status change notification is received.

        Args:
            status: The new status.
        """
        if self.status_change_callback:
            self.status_change_callback(status)


from .OPCUAMachine import SubscriptionWrapper
from .Duomix import Duomix
from .MixingpumpPlus import MixingpumpPlus

class DuomixPlus (Duomix, MixingpumpPlus):
    """
    DuomixPlus

    OPC-UA client class for m-tec Duo-Mix 3DCP+ machines (Mixingpump).
    Inherits from Duomix and MixingpumpPlus.
    """

    """Backward compatibility"""
    def startDosingpump(self):
        """
        DEPRECATED. Use '.dosingpump = True' instead.
        """
        self.dosingpump = True
    def stopDosingpump(self):
        """
        DEPRECATED. Use '.dosingpump = False' instead.
        """
        self.dosingpump = False
    def setSpeedDosingpump(self, speed):
        """
        DEPRECATED. Use '.dosingspeed = speed' instead.
        """
        self.dosingspeed = speed
    def setWater(self, speed):
        """
        DEPRECATED. Use '.water = speed' instead.
        """
        self.water = speed


from .MixingpumpPlus import MixingpumpPlus
from .Smp import Smp

class SmpPlus(Smp, MixingpumpPlus):
    """
    SMP

    OPC-UA client class for m-tec SMP machines (Mixingpump).
    Inherits from Smp and MixingpumpPlus.
    """


"""
mtecconnect3dcp

OPC-UA client classes for m-tec machines (Mixingpump, Printhead, Dosingpump).
"""
from .Smp import Smp
from .SmpPlus import SmpPlus
from .Duomix import Duomix
from .DuomixPlus import DuomixPlus
from .Printhead import Printhead
from .Dosingpump import Dosingpump
from .Pump import Pump


from .ModbusMachine import ModbusMachine

class Pump(ModbusMachine):
    """
    Class for controlling a pump via Modbus.
    Inherits from ModbusMachine.
    """
    # Class-level bit masks for status decoding
    _RUNNING_MASK = 0x0400
    _REVERSE_MASK = 0x0200

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._last_speed: float = 0.0
        self._last_running: bool = False
        self._last_reverse: bool = False

    @property
    def s_ready(self) -> bool:
        """
        bool: True if the machine is ready for operation (on, remote, mixer and mixingpump on).
        """
        switches = self.read("FD06")
        return ((switches % 32) - (switches % 16) != 0)

    @property
    def run(self) -> bool:
        """
        bool: True if the pump is set to run, False otherwise.
        """
        return (self.read("FA00") & self._RUNNING_MASK) != 0

    @run.setter
    def run(self, state: bool):
        """
        Set the running state of the pump.

        Args:
            state (bool): True to start, False to stop.
        """
        self._last_running = state
        if state:
            if self._last_reverse:
                v = self.write("FA00", 0xC600)
            else:
                v = self.write("FA00", 0xC400)
            self.keepalive()
            return v
        else:
            v = self.write("FA00", 0x0000)
            self.stop_keepalive()
            return v
    
    @property
    def s_pumping(self) -> bool:
        """
        bool: True if the pump is running, False otherwise.
        """
        return abs(self.m_speed) > 0
    
    @property
    def s_pumping_forward(self) -> bool:
        """
        bool: True if the pump is running forward, False otherwise.
        """
        return self.m_speed > 0
    
    @property
    def s_pumping_reverse(self) -> bool:
        """
        bool: True if the pump is running in reverse, False otherwise.
        """
        return self.m_speed < 0

    @property
    def reverse(self) -> bool:
        """
        bool: True if the pump is running in reverse, False otherwise.
        """
        return (self.read("FA00") & self._REVERSE_MASK) != 0

    @reverse.setter
    def reverse(self, state: bool):
        """
        Set the running direction of the pump.

        Args:
            state (bool): True for reverse, False for forward.
        """
        self._last_reverse = state
        if self._last_running:
            if state:
                return self.write("FA00", 0xC600)
            else:
                return self.write("FA00", 0xC400)
        else:
            return self.write("FA00", 0x0000)

    @property
    def _frequency(self) -> float:
        """
        float: Frequency of the pump in Hz.
        """
        return self.read("FD00") / 100

    @_frequency.setter
    def _frequency(self, value: float):
        """
        Set the frequency of the pump.

        Args:
            value (float): Frequency in Hz.
        """
        return self.write("FA01", int(value * 100))

    @property
    def m_voltage(self) -> float:
        """
        float: Voltage of the pump in V.
        """
        return self.read("FD05") / 100

    @property
    def m_current(self) -> float:
        """
        float: Current of the pump in A.
        """
        return self.read("FD03") / 100

    @property
    def m_torque(self) -> float:
        """
        float: Torque of the pump in Nm.
        """
        return self.read("FD18") / 100

    def emergency_stop(self):
        """
        Emergency stop for the pump.
        """
        return self.write("FA00", 0x1000)

    @property
    def m_speed(self) -> float:
        """
        float: Real speed of the pump in Hz. Negative values indicate reverse direction.
        """
        if self.reverse:
            return -self._frequency
        return self._frequency

    @property
    def speed(self) -> float:
        """
        float: Set speed of the pump. Negative values indicate reverse direction.
        """
        return self._last_speed

    @speed.setter
    def speed(self, value: float):
        """
        Set the speed of the pump.

        Args:
            value (float): Speed. Negative values indicate reverse direction.
        """
        if value == self._last_speed:
            return
        if value == 0:
            self.run = False
        else:
            if value < 0 and not self._last_speed < 0:
                self.reverse = True
            elif value > 0 and not self._last_speed > 0:
                self.reverse = False
            self._frequency = abs(value)
        self._last_speed = value


    """Backward compatibility"""
    def start(self):
        """
        DEPRECATED: Use '.run = True' instead.
        """
        self.reverse = False
        self.run = True
    def start_reverse(self):
        """
        DEPRECATED: Use '.reverse = True' and '.run = True' instead.
        """
        self.reverse = True
        self.run = True
    def stop(self):
        """
        DEPRECATED: Use '.run = False' instead.
        """
        self.run = False


from setuptools import setup, find_packages

setup(
    name="mtecconnect3dcp",
    version='v2.1.0',
    author="Paul Richter",
    author_email="paul.richter@m-tec.com",
    description="OPC-UA client classes for m-tec machines (Mixingpump, Printhead, Dosingpump)",
    long_description=open("README.md", encoding="utf-8").read() if __import__('os').path.exists("README.md") else "",
    long_description_content_type="text/markdown",
    url="https://github.com/m-tec-com/mtecConnect3DCP",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "asyncua>=1.0.0",
        "pyserial>=3.5",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    include_package_data=True,
    keywords="opcua m-tec connect 3dcp automation m2m mixingpump pump printhead dosingpump duo-mix flow-matic smp p20 p50 3d concrete printing",
    project_urls={
        "Source": "https://github.com/m-tec-com/mtecConnect3DCP",
        "Tracker": "https://github.com/m-tec-com/mtecConnect3DCP/issues",
    },
)


from .Mixingpump import Mixingpump

class Duomix (Mixingpump):
    """
    Duomix

    OPC-UA client class for m-tec Duo-Mix 3DCP+ machines (Mixingpump).
    Inherits from Mixingpump.
    """
    pass